from electricore.inputs.flux.modèles import FluxR151, FluxC15, FluxF1X
from electricore.inputs.flux.flux_relevés_Rxx import lire_flux_r151
from electricore.inputs.flux.flux_contrats_Cxx import lire_flux_c15
from electricore.inputs.flux.flux_factures_Fxx import lire_flux_f1x